/*
	RackAFX(TM)
	Applications Programming Interface
	Derived Class Object Definition
	Copyright(c) Tritone Systems Inc. 2006-2012

	Your plug-in must implement the constructor,
	destructor and virtual Plug-In API Functions below.
*/

#pragma once

// base class
#include "plugin.h"
#include "FFT.h"

// WP
#define FFT_LEN 2048
#define ONE_BIG_BUFFER

// abstract base class for RackAFX filters
class CAdaptiveNotch : public CPlugIn
{
public:
	// RackAFX Plug-In API Member Methods:
	// The followung 5 methods must be impelemented for a meaningful Plug-In
	//
	// 1. One Time Initialization
	CAdaptiveNotch();

	// 2. One Time Destruction
	virtual ~CAdaptiveNotch(void);

	// 3. The Prepare For Play Function is called just before audio streams
	virtual bool __stdcall prepareForPlay();

	// 4. processAudioFrame() processes an audio input to create an audio output
	virtual bool __stdcall processAudioFrame(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);

	// 5. userInterfaceChange() occurs when the user moves a control.
	virtual bool __stdcall userInterfaceChange(int nControlIndex);


	// OPTIONAL ADVANCED METHODS ------------------------------------------------------------------------------------------------
	// These are more advanced; see the website for more details
	//
	// 6. initialize() is called once just after creation; if you need to use Plug-In -> Host methods
	//				   such as sendUpdateGUI(), you must do them here and NOT in the constructor
	virtual bool __stdcall initialize();

	// 7. joystickControlChange() occurs when the user moves a control.
	virtual bool __stdcall joystickControlChange(float fControlA, float fControlB, float fControlC, float fControlD, float fACMix, float fBDMix);

	// 8. process buffers instead of Frames:
	// NOTE: set m_bWantBuffers = true to use this function
	virtual bool __stdcall processRackAFXAudioBuffer(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels, UINT uBufferSize);

	// 9. rocess buffers instead of Frames:
	// NOTE: set m_bWantVSTBuffers = true to use this function
	virtual bool __stdcall processVSTAudioBuffer(float** ppInputs, float** ppOutputs, UINT uNumChannels, int uNumFrames);

	// 10. MIDI Note On Event
	virtual bool __stdcall midiNoteOn(UINT uChannel, UINT uMIDINote, UINT uVelocity);

	// 11. MIDI Note Off Event
	virtual bool __stdcall midiNoteOff(UINT uChannel, UINT uMIDINote, UINT uVelocity, bool bAllNotesOff);


	// 12. MIDI Modulation Wheel uModValue = 0 -> 127
	virtual bool __stdcall midiModWheel(UINT uChannel, UINT uModValue);

	// 13. MIDI Pitch Bend
	//					nActualPitchBendValue = -8192 -> 8191, 0 is center, corresponding to the 14-bit MIDI value
	//					fNormalizedPitchBendValue = -1.0 -> +1.0, 0 is at center by using only -8191 -> +8191
	virtual bool __stdcall midiPitchBend(UINT uChannel, int nActualPitchBendValue, float fNormalizedPitchBendValue);

	// 14. MIDI Timing Clock (Sunk to BPM) function called once per clock
	virtual bool __stdcall midiClock();


	// 15. all MIDI messages -
	// NOTE: set m_bWantAllMIDIMessages true to get everything else (other than note on/off)
	virtual bool __stdcall midiMessage(unsigned char cChannel, unsigned char cStatus, unsigned char cData1, unsigned char cData2);

	// 16. initUI() is called only once from the constructor; you do not need to write or call it. Do NOT modify this function
	virtual bool __stdcall initUI();



	// Add your code here: ----------------------------------------------------------- //
	HANDLE m_hFFTThread;
	HANDLE m_hFFTStartEvent;
	HANDLE m_hFFTDoneEvent;

	// counter
	int m_nSampleCount;

	// audio double buffer
	double m_dAudioDoubleBuffer[FFT_LEN*2];

	// buffer process flag, will toggle true/false
	bool m_bProcessFFFTonBufferA;

	// FFT arrays
	double m_dFFT_In[FFT_LEN];
	double m_dFFT_OutRe[FFT_LEN];
	double m_dFFT_OutIm[FFT_LEN];
	double m_dFFT_Magnitude[FFT_LEN/2];

	// for the thread to call us to do FFT
	void doFFT();


	// notch filter
	CBiQuad m_LeftFilter;
	CBiQuad m_RightFilter;

	bool m_bStartFilter;

	// specific to the parametric EQ
	float m_c0; 
	float m_d0;

	void calculateFilterCoeffs(float fc, float Q);

	// returns -1 if failure
	//		 or Bin of max amplitude if success
	inline int normalizeBufferEx(double* pInputBuffer, UINT uBufferSize)
	{
		int nRet = -1;
		double fMax = 0;

		for(UINT j=0; j<uBufferSize; j++)
		{
			if((fabs(pInputBuffer[j])) > fMax)
			{
				nRet = j;
				fMax = fabs(pInputBuffer[j]);
			}
		}

		// normalize
		if(fMax > 0)
		{
			for(UINT j=0; j<uBufferSize; j++)
				pInputBuffer[j] = pInputBuffer[j]/fMax;
		}

		return nRet;
	}

	// END OF USER CODE -------------------------------------------------------------- //


	// ADDED BY RACKAFX -- DO NOT EDIT THIS CODE!!! ----------------------------------- //
	//  **--0x07FD--**

	float m_fNotchDepth;
	float m_fNotchQ;
	float m_f86HzMeterValue;
	float m_f172HzMeterValue;
	float m_f344HzMeterValue;
	float m_f688HzMeterValue;
	float m_f1376HzMeterValue;
	float m_f2752HzMeterValue;
	float m_f5504HzMeterValue;
	float m_f11008HzMeterValue;
	float m_f22050HzMeterValue;
	float m_f43HzMeterValue;

	// **--0x1A7F--**
	// ------------------------------------------------------------------------------- //

};





